@@ -21,6 +21,7 @@
 #include <atk/atk.h>
 #include <gtk/gtk.h>
 #include "gail.h"
+#include "gailfactory.h"
 
 static gboolean gail_focus_watcher      (GSignalInvocationHint *ihint,
                                          guint                  n_param_values,
@@ -30,13 +31,57 @@ static gboolean gail_select_watcher     (GSignalInvocationHint *ihint,
                                          guint                  n_param_values,
                                          const GValue          *param_values,
                                          gpointer               data);
+static gboolean gail_deselect_watcher   (GSignalInvocationHint *ihint,
+                                         guint                  n_param_values,
+                                         const GValue          *param_values,
+                                         gpointer               data);
 static AtkObject* gail_get_accessible_for_widget (GtkWidget    *widget,
                                                   gboolean     *transient);
+static void     gail_finish_select       (GtkWidget            *widget);
+static void     gail_map_cb              (GtkWidget            *widget);
+static gint     gail_focus_idle_handler  (gpointer             data);
+static void     gail_focus_notify        (GtkWidget            *widget);
 
 static void     gail_focus_tracker_init ();
 
 static GtkWidget* focus_widget = NULL;
-static GtkWidget *focus_before_menu = NULL;
+static GtkWidget* focus_before_menu = NULL;
+static guint focus_notify_handler = 0;    
+
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_WIDGET, gail_widget, gail_widget_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_CONTAINER, gail_container, gail_container_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_BUTTON, gail_button, gail_button_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_ITEM, gail_item, gail_item_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_MENU_ITEM, gail_menu_item, gail_menu_item_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_TOGGLE_BUTTON, gail_toggle_button, gail_toggle_button_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_IMAGE, gail_image, gail_image_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_TEXT_VIEW, gail_text_view, gail_text_view_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_COMBO, gail_combo, gail_combo_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_ENTRY, gail_entry, gail_entry_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_MENU_SHELL, gail_menu_shell, gail_menu_shell_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_MENU, gail_menu, gail_menu_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_WINDOW, gail_window, gail_window_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_RANGE, gail_range, gail_range_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_CLIST, gail_clist, gail_clist_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_LABEL, gail_label, gail_label_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_STATUSBAR, gail_statusbar, gail_statusbar_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_NOTEBOOK, gail_notebook, gail_notebook_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_CALENDAR, gail_calendar, gail_calendar_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_PROGRESS_BAR, gail_progress_bar, gail_progress_bar_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_SPIN_BUTTON, gail_spin_button, gail_spin_button_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_TREE_VIEW, gail_tree_view, gail_tree_view_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_FRAME, gail_frame, gail_frame_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_RADIO_BUTTON, gail_radio_button, gail_radio_button_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_ARROW, gail_arrow, gail_arrow_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_PIXMAP, gail_pixmap, gail_pixmap_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_SEPARATOR, gail_separator, gail_separator_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_BOX, gail_box, gail_box_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_SCROLLED_WINDOW, gail_scrolled_window, gail_scrolled_window_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_LIST, gail_list, gail_list_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_PANED, gail_paned, gail_paned_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_SCROLLBAR, gail_scrollbar, gail_scrollbar_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_OPTION_MENU, gail_option_menu, gail_option_menu_new)
+GAIL_ACCESSIBLE_FACTORY (GAIL_TYPE_CANVAS, gail_canvas, gail_canvas_new)
 
 static AtkObject*
 gail_get_accessible_for_widget (GtkWidget *widget,
@@ -82,7 +127,12 @@ gail_get_accessible_for_widget (GtkWidget *widget,
     }
   if (obj == NULL)
     {
+      AtkObject *focus_object;
+
       obj = gtk_widget_get_accessible (widget);
+      focus_object = g_object_get_data (G_OBJECT (obj), "gail-focus-object");
+      if (focus_object)
+        obj = focus_object;
     }
 
   return obj;
@@ -97,8 +147,6 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
   GObject *object;
   GtkWidget *widget;
   GdkEvent *event;
-  gboolean transient;
-  AtkObject *atk_obj;
 
   object = g_value_get_object (param_values + 0);
   g_return_val_if_fail (GTK_IS_WIDGET(object), FALSE);
@@ -172,13 +220,14 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
           return TRUE;
         }
     }
-  focus_widget = widget;
-
-  atk_obj  = gail_get_accessible_for_widget (widget, &transient);
-  atk_focus_tracker_notify (atk_obj);
-  if (transient)
-    g_object_unref (atk_obj);
-
+  /*
+   * The widget may not yet be visible on the screen so we wait until it is.
+   */
+  if (focus_notify_handler)
+    {
+      gtk_idle_remove (focus_notify_handler);
+    }
+  focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, widget);
   return TRUE; 
 }
 
@@ -189,7 +238,6 @@ gail_select_watcher (GSignalInvocationHint *ihint,
                      gpointer               data)
 {
   GObject *object;
-  AtkObject *aobject;
   GtkWidget *widget;
 
   object = g_value_get_object (param_values + 0);
@@ -198,34 +246,123 @@ gail_select_watcher (GSignalInvocationHint *ihint,
   widget = GTK_WIDGET (object);
 
   if (!GTK_WIDGET_MAPPED (widget))
-  /*
-   * select signal on non-mapped widget
-   * so ignore
-   */
-    return TRUE;
+    {
+      g_signal_connect (widget, "map",
+                        G_CALLBACK (gail_map_cb),
+                        NULL);
+      return TRUE;
+    }
 
+  gail_finish_select (widget);
+  return TRUE;
+}
+
+static void
+gail_finish_select (GtkWidget *widget)
+{
   if (focus_widget == widget)
   /*
    * We may already have got a notification because of bug 68483
    */
-    return TRUE;
-
+    return;
   
+  if (focus_notify_handler)
+    {
+      gtk_idle_remove (focus_notify_handler);
+      focus_notify_handler = 0;
+    }
   /*
-   * If previously focused widget is not a GtkMenuItem,
+   * If previously focused widget is not a GtkMenuItem or a GtkMenu,
    * keep track of it so we can return to it after menubar is deactivated
    */
-  if (!GTK_IS_MENU_ITEM(focus_widget))
-    focus_before_menu = focus_widget;
+  if (!GTK_IS_MENU_ITEM (focus_widget)  && !GTK_IS_MENU (focus_widget))
+    {
+      focus_before_menu = focus_widget;
+    } 
+  gail_focus_notify (widget);
+
+  return; 
+}
+
+static void
+gail_map_cb (GtkWidget *widget)
+{
+  gail_finish_select (widget);
+}
+
+static gboolean
+gail_deselect_watcher (GSignalInvocationHint *ihint,
+                       guint                  n_param_values,
+                       const GValue          *param_values,
+                       gpointer               data)
+{
+  GObject *object;
+  GtkWidget *widget;
+  GtkWidget *menu_shell;
+
+  object = g_value_get_object (param_values + 0);
+  g_return_val_if_fail (GTK_IS_WIDGET(object), FALSE);
+
+  widget = GTK_WIDGET (object);
+
   
-  focus_widget = widget;
-  aobject = gtk_widget_get_accessible (widget);
+  if (!GTK_IS_MENU_ITEM (widget))
+    return TRUE;
 
-  atk_focus_tracker_notify (aobject);
+  menu_shell = gtk_widget_get_parent (widget);
+  if (GTK_IS_MENU_SHELL (menu_shell))
+    {
+      GtkWidget *parent_menu_shell;
 
+      parent_menu_shell = GTK_MENU_SHELL (menu_shell)->parent_menu_shell;
+      if (GTK_IS_MENU (parent_menu_shell))
+        {
+          GtkWidget *active_menu_item;
+
+          active_menu_item = GTK_MENU_SHELL (parent_menu_shell)->active_menu_item;
+          if (active_menu_item)
+            {
+              focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, active_menu_item);
+            }
+        }
+    }
   return TRUE; 
 }
 
+static gint
+gail_focus_idle_handler (gpointer data)
+{
+  focus_notify_handler = 0;
+  gail_focus_notify (GTK_WIDGET (data));
+
+  return FALSE; 
+}
+
+static void
+gail_focus_notify (GtkWidget *widget)
+{
+  AtkObject *atk_obj;
+  gboolean transient;
+
+  focus_widget = widget;
+  if (focus_before_menu == focus_widget)
+    {
+      /*
+       * The UI may not have been updated yet; e.g. in gtkhtml2
+       * html_view_layout() is called in a idle handler
+       */
+      focus_before_menu = NULL;
+      focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, focus_widget);
+    }
+  else
+    {
+      atk_obj  = gail_get_accessible_for_widget (focus_widget, &transient);
+      atk_focus_tracker_notify (atk_obj);
+      if (transient)
+        g_object_unref (atk_obj);
+    }
+}
+
 static gboolean
 gail_deactivate_watcher (GSignalInvocationHint *ihint,
                     guint                  n_param_values,
@@ -235,23 +372,25 @@ gail_deactivate_watcher (GSignalInvocationHint *ihint,
   GObject *object;
   GtkWidget *widget;
   GtkMenuShell *shell;
-  gboolean transient;
-  AtkObject *atk_obj;
 
   object = g_value_get_object (param_values + 0);
   g_return_val_if_fail (GTK_IS_WIDGET(object), FALSE);
   widget = GTK_WIDGET (object);
 
+  if (focus_notify_handler)
+    {
+      gtk_idle_remove (focus_notify_handler);
+      focus_notify_handler = 0;
+    }
   g_return_val_if_fail (GTK_IS_MENU_SHELL(widget), TRUE);
   shell = GTK_MENU_SHELL(widget);
   if (shell->parent_menu_shell)
     return TRUE;
-  focus_widget = focus_before_menu;
 
-  atk_obj  = gail_get_accessible_for_widget (focus_widget, &transient);
-  atk_focus_tracker_notify (atk_obj);
-  if (transient)
-    g_object_unref (atk_obj);
+  if (focus_before_menu)
+  {
+    focus_notify_handler = gtk_idle_add (gail_focus_idle_handler, focus_before_menu);
+  }
 
   return TRUE; 
 }
@@ -287,9 +426,18 @@ gail_focus_tracker_init ()
              g_signal_lookup ("select", GTK_TYPE_ITEM), 0,
              gail_select_watcher, NULL, (GDestroyNotify) NULL);
 
+      /*
+       * A "deselect" signal is emitted when arrow key is used to
+       * move from a menu item in a menu to the parent menu.
+       */
+      g_signal_add_emission_hook (
+             g_signal_lookup ("deselect", GTK_TYPE_ITEM), 0,
+             gail_deselect_watcher, NULL, (GDestroyNotify) NULL);
+
       /*
        * We listen for deactivate signals on menushells to determine
-       * when the "focus" has left the menus.  */
+       * when the "focus" has left the menus.
+       */
       g_signal_add_emission_hook (
              g_signal_lookup ("deactivate", GTK_TYPE_MENU_SHELL), 0,
              gail_deactivate_watcher, NULL, (GDestroyNotify) NULL);
@@ -307,6 +455,69 @@ extern void gnome_accessibility_module_shutdown (void);
 
 static int gail_initialized = FALSE;
 
+static void
+gail_accessibility_module_init (void)
+{
+  if (gail_initialized)
+    {
+      return;
+    }
+  gail_initialized = TRUE;
+
+  g_print ("GTK Accessibility Module initialized\n");
+
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_WIDGET, gail_widget);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CONTAINER, gail_container);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_BUTTON, gail_button);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_ITEM, gail_item);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_MENU_ITEM, gail_menu_item);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_TOGGLE_BUTTON, gail_toggle_button);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_IMAGE, gail_image);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_TEXT_VIEW, gail_text_view);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_COMBO, gail_combo);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_ENTRY, gail_entry);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_MENU_BAR, gail_menu_shell);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_MENU, gail_menu);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_WINDOW, gail_window);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_HANDLE_BOX, gail_window);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_RANGE, gail_range);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CLIST, gail_clist);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_LABEL, gail_label);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_STATUSBAR, gail_statusbar);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_NOTEBOOK, gail_notebook);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CALENDAR, gail_calendar);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_PROGRESS_BAR, gail_progress_bar);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_SPIN_BUTTON, gail_spin_button);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_TREE_VIEW, gail_tree_view);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_FRAME, gail_frame);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CELL_RENDERER_TEXT, gail_text_cell);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CELL_RENDERER_TOGGLE, gail_boolean_cell);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_CELL_RENDERER_PIXBUF, gail_image_cell);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_RADIO_BUTTON, gail_radio_button);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_ARROW, gail_arrow);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_PIXMAP, gail_pixmap);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_SEPARATOR, gail_separator);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_BOX, gail_box);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_SCROLLED_WINDOW, gail_scrolled_window);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_LIST, gail_list);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_PANED, gail_paned);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_SCROLLBAR, gail_scrollbar);
+  GAIL_WIDGET_SET_FACTORY (GTK_TYPE_OPTION_MENU, gail_option_menu);
+
+  /* LIBGNOMECANVAS SUPPORT */
+  GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS, gail_canvas);
+  GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS_GROUP, gail_canvas_group);
+  GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS_TEXT, gail_canvas_text);
+  GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS_RICH_TEXT, gail_canvas_text);
+  GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS_WIDGET, gail_canvas_widget);
+  GAIL_WIDGET_SET_FACTORY (GNOME_TYPE_CANVAS_ITEM, gail_canvas_item);
+
+  atk_focus_tracker_init (gail_focus_tracker_init);
+
+  /* Initialize the GailUtility class */
+  g_type_class_unref (g_type_class_ref (GAIL_TYPE_UTIL));
+}
+
 /**
  * gnome_accessibility_module_init:
  * @void: 
@@ -317,110 +528,7 @@ static int gail_initialized = FALSE;
 void
 gnome_accessibility_module_init (void)
 {
-  AtkRegistry *default_registry;
-
-  if (gail_initialized)
-    {
-      return;
-    }
-  gail_initialized = TRUE;
-
-  g_print ("GTK Accessibility Module initialized\n");
-
-  default_registry = atk_get_default_registry ();
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_WIDGET, 
-                                 GAIL_TYPE_WIDGET_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_CONTAINER, 
-                                 GAIL_TYPE_CONTAINER_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_BUTTON, 
-                                 GAIL_TYPE_BUTTON_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_ITEM, 
-                                 GAIL_TYPE_ITEM_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_MENU_ITEM, 
-                                 GAIL_TYPE_MENU_ITEM_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_TOGGLE_BUTTON, 
-                                 GAIL_TYPE_TOGGLE_BUTTON_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_IMAGE, 
-                                 GAIL_TYPE_IMAGE_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_TEXT_VIEW, 
-                                 GAIL_TYPE_TEXT_VIEW_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_COMBO, 
-                                 GAIL_TYPE_COMBO_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_ENTRY, 
-                                 GAIL_TYPE_ENTRY_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_MENU_BAR, 
-                                 GAIL_TYPE_MENU_SHELL_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_MENU,
-                                 GAIL_TYPE_MENU_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_WINDOW, 
-                                 GAIL_TYPE_WINDOW_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_HANDLE_BOX, 
-                                 GAIL_TYPE_WINDOW_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_RANGE,
-                                 GAIL_TYPE_RANGE_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_CLIST, 
-                                 GAIL_TYPE_CLIST_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_LABEL, 
-                                 GAIL_TYPE_LABEL_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_STATUSBAR, 
-                                 GAIL_TYPE_STATUSBAR_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_NOTEBOOK, 
-                                 GAIL_TYPE_NOTEBOOK_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_CALENDAR, 
-                                 GAIL_TYPE_CALENDAR_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_PROGRESS_BAR,
-                                 GAIL_TYPE_PROGRESS_BAR_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_SPIN_BUTTON,
-                                 GAIL_TYPE_SPIN_BUTTON_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_TREE_VIEW,
-                                 GAIL_TYPE_TREE_VIEW_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_FRAME,
-                                 GAIL_TYPE_FRAME_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_CELL_RENDERER_TEXT,
-                                 GAIL_TYPE_TEXT_CELL_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_CELL_RENDERER_TOGGLE,
-                                 GAIL_TYPE_BOOLEAN_CELL_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_CELL_RENDERER_PIXBUF,
-                                 GAIL_TYPE_IMAGE_CELL_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_RADIO_BUTTON,
-                                 GAIL_TYPE_RADIO_BUTTON_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_ARROW,
-                                 GAIL_TYPE_ARROW_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_PIXMAP,
-                                 GAIL_TYPE_PIXMAP_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_SEPARATOR,
-                                 GAIL_TYPE_SEPARATOR_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_BOX,
-                                 GAIL_TYPE_BOX_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_SCROLLED_WINDOW,
-                                 GAIL_TYPE_SCROLLED_WINDOW_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_LIST,
-                                 GAIL_TYPE_LIST_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_PANED,
-                                 GAIL_TYPE_PANED_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_SCROLLBAR,
-                                 GAIL_TYPE_SCROLLBAR_FACTORY);
-  atk_registry_set_factory_type (default_registry, GTK_TYPE_OPTION_MENU,
-                                 GAIL_TYPE_OPTION_MENU_FACTORY);
-
-  /* LIBGNOMECANVAS SUPPORT */
-  atk_registry_set_factory_type (default_registry, GNOME_TYPE_CANVAS,
-				GAIL_TYPE_CANVAS_FACTORY);
-  atk_registry_set_factory_type (default_registry, GNOME_TYPE_CANVAS_GROUP,
-				 GAIL_TYPE_CANVAS_GROUP_FACTORY);
-  atk_registry_set_factory_type (default_registry, GNOME_TYPE_CANVAS_TEXT,
-				 GAIL_TYPE_CANVAS_TEXT_FACTORY);
-  atk_registry_set_factory_type (default_registry, GNOME_TYPE_CANVAS_RICH_TEXT,
-				 GAIL_TYPE_CANVAS_TEXT_FACTORY);
-  atk_registry_set_factory_type (default_registry, GNOME_TYPE_CANVAS_WIDGET,
-				 GAIL_TYPE_CANVAS_WIDGET_FACTORY);
-  atk_registry_set_factory_type (default_registry, GNOME_TYPE_CANVAS_ITEM,
-				 GAIL_TYPE_CANVAS_ITEM_FACTORY);
-
-  atk_focus_tracker_init (gail_focus_tracker_init);
-
-  /* Initialize the GailUtility class */
-  g_type_class_unref (g_type_class_ref (GAIL_TYPE_UTIL));
+  gail_accessibility_module_init ();
 }
 
 /**
@@ -447,7 +555,7 @@ gnome_accessibility_module_shutdown (void)
 int
 gtk_module_init (gint *argc, char** argv[])
 {
-  gnome_accessibility_module_init ();
+  gail_accessibility_module_init ();
 
   return 0;
 }
