@@ -33,8 +33,8 @@ static AtkStateSet*          gail_button_ref_state_set    (AtkObject       *obj)
 static void                  gail_button_real_notify_gtk  (GObject         *obj,
                                                            GParamSpec      *pspec);
 
-static void                  gail_button_real_init        (GailWidget      *widget,
-                                                           GtkWidget       *gtk_widget);
+static void                  gail_button_real_initialize  (AtkObject       *obj,
+                                                           gpointer        data);
 static void                  gail_button_finalize         (GObject        *object);
 
 static void                  gail_button_pressed_enter_handler  (GtkWidget       *widget);
@@ -129,14 +129,14 @@ gail_button_class_init (GailButtonClass *klass)
 
   gobject_class->finalize = gail_button_finalize;
 
-  parent_class = g_type_class_ref (GAIL_TYPE_CONTAINER);
+  parent_class = g_type_class_peek_parent (klass);
 
   class->get_name = gail_button_get_name;
   class->get_n_children = gail_button_get_n_children;
   class->ref_child = gail_button_ref_child;
   class->ref_state_set = gail_button_ref_state_set;
+  class->initialize = gail_button_real_initialize;
 
-  widget_class->init = gail_button_real_init;
   widget_class->notify_gtk = gail_button_real_notify_gtk;
 }
 
@@ -159,9 +159,9 @@ gail_button_new (GtkWidget *widget)
 
   object = g_object_new (GAIL_TYPE_BUTTON, NULL);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
   accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
+
   accessible->role = ATK_ROLE_PUSH_BUTTON;
 
   return accessible;
@@ -201,35 +201,38 @@ gail_button_get_name (AtkObject *obj)
 }
 
 static void
-gail_button_real_init (GailWidget         *widget,
-                       GtkWidget          *gtk_widget)
+gail_button_real_initialize (AtkObject *obj,
+                             gpointer   data)
 {
-  GailContainer *container = GAIL_CONTAINER (widget);
-  GailButton *button = GAIL_BUTTON (widget);
-  GAIL_WIDGET_CLASS (parent_class)->init (widget, gtk_widget);
+  GailButton *button = GAIL_BUTTON (obj);
+  guint handler_id;
+
+  ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
   /*
    * As we report the button as having no children we are not interested
    * in add and remove signals
    */
-  g_signal_handler_disconnect (gtk_widget, container->add_handler);
-  g_signal_handler_disconnect (gtk_widget, container->remove_handler);
+  handler_id = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (obj), "gail-add-handler-id"));
+  g_signal_handler_disconnect (data, handler_id);
+  handler_id = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT(obj), "gail-remove-handler-id"));
+  g_signal_handler_disconnect (data, handler_id);
 
   button->state = GTK_STATE_NORMAL;
 
-  g_signal_connect (gtk_widget,
+  g_signal_connect (data,
                     "pressed",
                     G_CALLBACK (gail_button_pressed_enter_handler),
                     NULL);
-  g_signal_connect (gtk_widget,
+  g_signal_connect (data,
                     "enter",
                     G_CALLBACK (gail_button_pressed_enter_handler),
                     NULL);
-  g_signal_connect (G_OBJECT (gtk_widget),
+  g_signal_connect (data,
                     "released",
                     G_CALLBACK (gail_button_released_leave_handler),
                     NULL);
-  g_signal_connect (G_OBJECT (gtk_widget),
+  g_signal_connect (data,
                     "leave",
                     G_CALLBACK (gail_button_released_leave_handler),
                     NULL);
@@ -288,6 +291,9 @@ gail_button_do_action (AtkAction *action,
      */
     return FALSE;
 
+  if (!GTK_WIDGET_SENSITIVE (widget) || !GTK_WIDGET_VISIBLE (widget))
+    return FALSE;
+
   button = GTK_BUTTON (widget); 
   switch (i)
     {
@@ -696,14 +702,22 @@ get_label_from_button (GtkWidget *button)
   if (GTK_IS_BOX (child))
     {
       /*
-       * Child is not a label; perhaps the button is for a stock item.
-       * If it is, then the second child of the child should be a label.
+       * Child is not a label.  Search for a label in the box's children.
        */
       GList *children, *tmp_list;
  
       children = gtk_container_children (GTK_CONTAINER (child));
-      tmp_list = g_list_nth (children, 1);
-      child = GTK_WIDGET (tmp_list->data);
+
+      child = NULL;
+      for (tmp_list = children; tmp_list != NULL; tmp_list = tmp_list->next) 
+	{
+	  if (GTK_IS_LABEL (tmp_list->data))
+	    {
+	      child = GTK_WIDGET (tmp_list->data);
+	      break;
+	    }  
+	}
+
       g_list_free (children);
     }
   return child;
