@@ -17,6 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <math.h>
 #include <gtk/gtk.h>
 #include <libgnomecanvas/gnome-canvas.h>
 #include <libgnomecanvas/gnome-canvas-util.h>
@@ -281,12 +282,13 @@ gail_canvas_item_get_extents (AtkComponent *component,
   atk_gobj = ATK_GOBJECT_ACCESSIBLE (component);
   obj = atk_gobject_accessible_get_object (atk_gobj);
 
-  /* Get the GnomeCanvasItem */
-  item = GNOME_CANVAS_ITEM (obj);
-  if (item == NULL)
+  if (obj == NULL) 
     /* item is defunct */
     return;
 
+  /* Get the GnomeCanvasItem */
+  item = GNOME_CANVAS_ITEM (obj);
+
   /* If this item has no parent canvas, something's broken */
   g_return_if_fail (GTK_IS_WIDGET (item->canvas));
 
@@ -356,22 +358,109 @@ get_item_extents (GnomeCanvasItem   *item,
                   gint              *width,
                   gint              *height)
 {
-  gdouble d_x1, d_y1, d_x2, d_y2;
-  gdouble win_x1, win_y1, win_x2, win_y2;
-  gint i_x, i_y;
+  double bx1, by1, bx2, by2;
+  double i2c[6];
+  ArtPoint p1, p2, p3, p4;
+  ArtPoint q1, q2, q3, q4;
+  double min_x1, min_y1, min_x2, min_y2;
+  double max_x1, max_y1, max_x2, max_y2;
+  int x1, y1, x2, y2;
+  int scroll_x, scroll_y;
 
-  gnome_canvas_item_get_bounds (item, &d_x1, &d_y1, &d_x2, &d_y2);
-  /* Convert them to window-based coordinates */
-  gnome_canvas_world_to_window (item->canvas, d_x1, d_y1, &win_x1, &win_y1);
-  gnome_canvas_world_to_window (item->canvas, d_x2, d_y2, &win_x2, &win_y2);
+  /* Get the bounding box in item-relative coordinates */
 
-  *width = (gint) (win_x2 - win_x1);
-  *height = (gint) (win_y2 - win_y1);
+  bx1 = by1 = bx2 = by2 = 0.0;
 
-  gnome_canvas_get_scroll_offsets (item->canvas, &i_x, &i_y);
+  if (GNOME_CANVAS_ITEM_CLASS (G_OBJECT_GET_CLASS (item))->bounds)
+    (* GNOME_CANVAS_ITEM_CLASS (G_OBJECT_GET_CLASS (item))->bounds) (item, &bx1, &by1, &bx2, &by2);
 
-  *x = (gint) win_x1 - i_x;
-  *y = (gint) win_y1 - i_y;
+  /* Get the item coordinates -> canvas pixel coordinates affine */
+
+  gnome_canvas_item_i2c_affine (item, i2c);
+
+  /* Convert the bounding box to canvas pixel coordinates and find its minimum
+   * surrounding rectangle.
+   */
+  
+  p1.x = p2.x = bx1;
+  p1.y = p4.y = by1;
+  p3.x = p4.x = bx2;
+  p2.y = p3.y = by2;
+
+  art_affine_point (&q1, &p1, i2c);
+  art_affine_point (&q2, &p2, i2c);
+  art_affine_point (&q3, &p3, i2c);
+  art_affine_point (&q4, &p4, i2c);
+
+  if (q1.x < q2.x)
+    {
+      min_x1 = q1.x;
+      max_x1 = q2.x;
+    }
+  else
+    {
+      min_x1 = q2.x;
+      max_x1 = q1.x;
+    }
+
+  if (q1.y < q2.y)
+    {
+      min_y1 = q1.y;
+      max_y1 = q2.y;
+    }
+  else
+    {
+      min_y1 = q2.y;
+      max_y1 = q1.y;
+    }
+
+  if (q3.x < q4.x)
+    {
+      min_x2 = q3.x;
+      max_x2 = q4.x;
+    }
+  else
+    {
+      min_x2 = q4.x;
+      max_x2 = q3.x;
+    }
+
+  if (q3.y < q4.y)
+    {
+      min_y2 = q3.y;
+      max_y2 = q4.y;
+    }
+  else
+    {
+      min_y2 = q4.y;
+      max_y2 = q3.y;
+    }
+
+  bx1 = MIN (min_x1, min_x2);
+  by1 = MIN (min_y1, min_y2);
+  bx2 = MAX (max_x1, max_x2);
+  by2 = MAX (max_y1, max_y2);
+
+  /* Convert to integer coordinates */
+
+  x1 = floor (bx1);
+  y1 = floor (by1);
+  x2 = ceil (bx2);
+  y2 = ceil (by2);
+
+  gnome_canvas_get_scroll_offsets (item->canvas, &scroll_x, &scroll_y);
+
+  if (x)
+    *x = x1 + scroll_x;
+
+  if (y)
+    *y = y1 + scroll_y;
+
+  if (width)
+    *width = x2 - x1;
+
+  if (height)
+    *height = y2 - y1;
 }
 
 static gboolean
