@@ -22,7 +22,6 @@
 #include "gailclist.h"
 #include "gailclistcell.h"
 #include "gailcellparent.h"
-#include "gailclistfactory.h"
 
 /* Copied from gtkclist.c */
 /* this defigns the base grid spacing */
@@ -814,7 +813,7 @@ gail_clist_get_column_header (AtkTable      *table,
   if (return_widget == NULL)
     return NULL;
 
-  g_return_if_fail (GTK_IS_BIN (return_widget));
+  g_return_val_if_fail (GTK_IS_BIN (return_widget), NULL);
   return_widget = gtk_bin_get_child (GTK_BIN(return_widget));
 
   return gtk_widget_get_accessible (return_widget);
