@@ -20,7 +20,6 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include "gailnotebook.h"
-#include "gailnotebookfactory.h"
 #include "gailnotebookpage.h"
 
 static void         gail_notebook_class_init          (GailNotebookClass *klass);
@@ -44,6 +43,9 @@ static gboolean     gail_notebook_is_child_selected   (AtkSelection   *selection
                                                        gint           i);
 static AtkObject*   find_child_in_list                (GList          *list,
                                                        gint           index);
+static gboolean     gail_notebook_focus_cb            (GtkWidget      *widget,
+                                                       GtkDirectionType type);
+static gboolean     gail_notebook_check_focus_tab     (gpointer       data);
 
 static gpointer parent_class = NULL;
 
@@ -112,6 +114,7 @@ gail_notebook_object_init (GailNotebook      *notebook)
 {
   notebook->page_cache = NULL;
   notebook->selected_page = -1;
+  notebook->focus_tab_page = -1;
 }
 
 AtkObject* 
@@ -199,6 +202,14 @@ gail_notebook_real_initialize (AtkObject *obj,
       page = gtk_notebook_get_nth_page (gtk_notebook, ++i);
     }
   notebook->selected_page = gtk_notebook_get_current_page (gtk_notebook);
+  if (gtk_notebook->focus_tab && gtk_notebook->focus_tab->data)
+    {
+      notebook->focus_tab_page = g_list_index (gtk_notebook->children, gtk_notebook->focus_tab->data);
+    }
+  g_signal_connect (gtk_notebook,
+                    "focus",
+                    G_CALLBACK (gail_notebook_focus_cb),
+                    NULL);
 }
 
 static void
@@ -211,14 +222,19 @@ gail_notebook_real_notify_gtk (GObject           *obj,
   if (strcmp (pspec->name, "page") == 0)
     {
       gint page_num, old_page_num;
+      gint focus_page_num, old_focus_page_num;
       GailNotebook *gail_notebook = GAIL_NOTEBOOK (atk_obj);
+      GtkNotebook *gtk_notebook = GTK_NOTEBOOK (widget);
      
       /*
        * Notify SELECTED state change for old and new page
        */
       old_page_num = gail_notebook->selected_page;
-      page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (widget));
+      page_num = gtk_notebook_get_current_page (gtk_notebook);
       gail_notebook->selected_page = page_num;
+      old_focus_page_num = gail_notebook->focus_tab_page;
+      focus_page_num = g_list_index (gtk_notebook->children, gtk_notebook->focus_tab->data);
+      gail_notebook->focus_tab_page = focus_page_num;
     
       if (page_num != old_page_num)
         {
@@ -242,9 +258,16 @@ gail_notebook_real_notify_gtk (GObject           *obj,
                * The page which is being displayed has changed so tell the focus
                * tracker
                */
-              atk_focus_tracker_notify (atk_obj);
+              atk_focus_tracker_notify (obj);
             }
         }
+      else if (focus_page_num != old_focus_page_num)
+        {
+          AtkObject *obj;
+
+          obj = find_child_in_list (gail_notebook->page_cache, focus_page_num);
+          atk_focus_tracker_notify (obj);
+        }
     }
   else
     GAIL_WIDGET_CLASS (parent_class)->notify_gtk (obj, pspec);
@@ -409,3 +432,42 @@ find_child_in_list (GList *list,
     }
   return obj;
 }
+
+static gboolean
+gail_notebook_focus_cb (GtkWidget      *widget,
+                        GtkDirectionType type)
+{
+  switch (type)
+    {
+    case GTK_DIR_LEFT:
+    case GTK_DIR_RIGHT:
+      gtk_idle_add (gail_notebook_check_focus_tab, widget);
+      break;
+    default:
+      break;
+    }
+  return FALSE;
+}
+
+static gboolean
+gail_notebook_check_focus_tab (gpointer data)
+{
+  GtkWidget *widget = GTK_WIDGET (data);
+  AtkObject* atk_obj = gtk_widget_get_accessible (widget);
+  gint focus_page_num, old_focus_page_num;
+  GailNotebook *gail_notebook = GAIL_NOTEBOOK (atk_obj);
+  GtkNotebook *gtk_notebook = GTK_NOTEBOOK (widget);
+
+  old_focus_page_num = gail_notebook->focus_tab_page;
+  focus_page_num = g_list_index (gtk_notebook->children, gtk_notebook->focus_tab->data);
+  gail_notebook->focus_tab_page = focus_page_num;
+  if (old_focus_page_num != focus_page_num)
+    {
+      AtkObject *obj;
+
+      obj = find_child_in_list (gail_notebook->page_cache, focus_page_num);
+      atk_focus_tracker_notify (obj);
+    }
+  return FALSE;
+}
+
